@@ -194,6 +194,7 @@ def create_view(
     engine: sa.engine.Engine,
     models: Callable,
     fetchall: Callable,
+    index: str,
     schema: str,
     tables: Set,
     user_defined_fkey_tables: dict,
@@ -213,28 +214,36 @@ def create_view(
     So if 'specie' was the only row before, and the next query returns
     'unit' and 'structure', we want to end up with the result below.
 
-    table_name | primary_keys | foreign_keys
-    -----------+--------------+--------------
-    specie     | {id}         | {id, user_id}
-    unit       | {id}         | {id, profile_id}
-    structure  | {id}         | {id}
+     table_name | primary_keys | foreign_keys     | indices
+    ------------+--------------+------------------+------------
+     specie     | {id}         | {id, user_id}    | {foo, bar}
+     unit       | {id}         | {id, profile_id} | {foo, bar}
+     structure  | {id}         | {id}             | {foo, bar}
+     unit       | {id}         | {id, profile_id} | {foo, bar}
+     structure  | {id}         | {id}             | {foo, bar}
     """
 
     rows: dict = {}
     if MATERIALIZED_VIEW in views:
-        for table_name, primary_keys, foreign_keys in fetchall(
+        for table_name, primary_keys, foreign_keys, indices in fetchall(
             sa.select(["*"]).select_from(
                 sa.text(f"{schema}.{MATERIALIZED_VIEW}")
             )
         ):
             rows.setdefault(
                 table_name,
-                {"primary_keys": set(), "foreign_keys": set()},
+                {
+                    "primary_keys": set(),
+                    "foreign_keys": set(),
+                    "indices": set(),
+                },
             )
             if primary_keys:
                 rows[table_name]["primary_keys"] = set(primary_keys)
             if foreign_keys:
                 rows[table_name]["foreign_keys"] = set(foreign_keys)
+            if indices:
+                rows[table_name]["indices"] = set(indices)
 
         engine.execute(DropView(schema, MATERIALIZED_VIEW))
 
@@ -245,32 +254,39 @@ def create_view(
     for table_name, columns in fetchall(_primary_keys(models, schema, tables)):
         rows.setdefault(
             table_name,
-            {"primary_keys": set(), "foreign_keys": set()},
+            {"primary_keys": set(), "foreign_keys": set(), "indices": set()},
         )
         if columns:
             rows[table_name]["primary_keys"] |= set(columns)
+            rows[table_name]["indices"] |= set([index])
 
     for table_name, columns in fetchall(_foreign_keys(models, schema, tables)):
         rows.setdefault(
             table_name,
-            {"primary_keys": set(), "foreign_keys": set()},
+            {"primary_keys": set(), "foreign_keys": set(), "indices": set()},
         )
         if columns:
             rows[table_name]["foreign_keys"] |= set(columns)
+            rows[table_name]["indices"] |= set([index])
 
     if user_defined_fkey_tables:
         for table_name, columns in user_defined_fkey_tables.items():
             rows.setdefault(
                 table_name,
-                {"primary_keys": set(), "foreign_keys": set()},
+                {
+                    "primary_keys": set(),
+                    "foreign_keys": set(),
+                    "indices": set(),
+                },
             )
             if columns:
                 rows[table_name]["foreign_keys"] |= set(columns)
+                rows[table_name]["indices"] |= set([index])
 
     if not rows:
         rows.setdefault(
             None,
-            {"primary_keys": set(), "foreign_keys": set()},
+            {"primary_keys": set(), "foreign_keys": set(), "indices": set()},
         )
 
     statement = sa.select(
@@ -278,6 +294,7 @@ def create_view(
             sa.column("table_name"),
             sa.column("primary_keys"),
             sa.column("foreign_keys"),
+            sa.column("indices"),
         )
         .data(
             [
@@ -289,6 +306,9 @@ def create_view(
                     array(fields.get("foreign_keys"))
                     if fields.get("foreign_keys")
                     else None,
+                    array(fields.get("indices"))
+                    if fields.get("indices")
+                    else None,
                 )
                 for table_name, fields in rows.items()
             ]
@@ -299,7 +319,12 @@ def create_view(
     engine.execute(CreateView(schema, MATERIALIZED_VIEW, statement))
     engine.execute(DropIndex("_idx"))
     engine.execute(
-        CreateIndex("_idx", schema, MATERIALIZED_VIEW, ["table_name"])
+        CreateIndex(
+            "_idx",
+            schema,
+            MATERIALIZED_VIEW,
+            ["table_name"],
+        )
     )
     logger.debug(f"Created view: {schema}.{MATERIALIZED_VIEW}")
 
