@@ -110,7 +110,43 @@ class TestBase(object):
 
     def test_indices(self, connection):
         pg_base = Base(connection.engine.url.database)
-        assert pg_base.indices("book", "public") == []
+        assert pg_base.indices("contact_item", "public") == [
+            {
+                "name": "contact_item_contact_id_key",
+                "unique": True,
+                "column_names": ["contact_id"],
+                "include_columns": [],
+                "duplicates_constraint": "contact_item_contact_id_key",
+                "dialect_options": {"postgresql_include": []},
+            },
+            {
+                "name": "contact_item_name_key",
+                "unique": True,
+                "column_names": ["name"],
+                "include_columns": [],
+                "duplicates_constraint": "contact_item_name_key",
+                "dialect_options": {"postgresql_include": []},
+            },
+        ]
+
+    def test_columns(self, connection):
+        pg_base = Base(connection.engine.url.database)
+        assert pg_base.columns("public", "book") == [
+            "buyer_id",
+            "copyright",
+            "description",
+            "isbn",
+            "publisher_id",
+            "seller_id",
+            "tags",
+            "title",
+        ]
+        assert pg_base.columns("public", "shelf") == ["id", "shelf"]
+        assert pg_base.columns("public", "book_author") == [
+            "author_id",
+            "book_isbn",
+            "id",
+        ]
 
     @patch("pgsync.base.logger")
     @patch("pgsync.sync.Base.execute")
