@@ -10,9 +10,10 @@ from pgsync.exc import (
     TableNotInNodeError,
 )
 from pgsync.settings import NTHREADS_POLLDB
+from pgsync.singleton import Singleton
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, noop, search, sort_list
+from .testing_utils import assert_resync_empty, noop, search, sort_list
 
 
 @pytest.mark.usefixtures("table_creator")
@@ -60,6 +61,7 @@ class TestRoot(object):
             f"{sync.database}_testdb",
             upto_nchanges=None,
         )
+        Singleton._instances = {}
 
         yield books
 
@@ -414,6 +416,7 @@ class TestRoot(object):
         """
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
@@ -456,6 +459,7 @@ class TestRoot(object):
         """Test sync updates primary_key and then sync in concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
@@ -521,6 +525,7 @@ class TestRoot(object):
         """Test sync insert and then sync in non-concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
@@ -561,6 +566,7 @@ class TestRoot(object):
         """Test sync update and then sync in non-concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
@@ -601,6 +607,7 @@ class TestRoot(object):
         """Test sync update and then sync in concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
@@ -662,6 +669,7 @@ class TestRoot(object):
         """Test sync delete and then sync in concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
