@@ -14,9 +14,10 @@ from pgsync.exc import (
 )
 from pgsync.node import Tree
 from pgsync.settings import NTHREADS_POLLDB
+from pgsync.singleton import Singleton
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, noop, search, sort_list
+from .testing_utils import assert_resync_empty, noop, search, sort_list
 
 
 @pytest.mark.usefixtures("table_creator")
@@ -56,6 +57,7 @@ class TestParentSingleChildFkOnParent(object):
             f"{sync.database}_testdb",
             upto_nchanges=None,
         )
+        Singleton._instances = {}
 
         yield books
 
@@ -595,6 +597,7 @@ class TestParentSingleChildFkOnParent(object):
         """Test sync updates primary_key then sync in non-concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {
                 "table": "book",
                 "columns": ["isbn", "title"],
@@ -690,6 +693,7 @@ class TestParentSingleChildFkOnParent(object):
         """Test sync updates primary_key and then sync in concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {
                 "table": "book",
                 "columns": ["isbn", "title"],
@@ -802,6 +806,7 @@ class TestParentSingleChildFkOnParent(object):
         """Test sync insert and then sync in non-concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {
                 "table": "book",
                 "columns": ["isbn", "title"],
@@ -901,6 +906,7 @@ class TestParentSingleChildFkOnParent(object):
         """Test sync update and then sync in non-concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {
                 "table": "book",
                 "columns": ["isbn", "title"],
@@ -990,6 +996,7 @@ class TestParentSingleChildFkOnParent(object):
         """Test sync update and then sync in concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {
                 "table": "book",
                 "columns": ["isbn", "title"],
@@ -1094,6 +1101,7 @@ class TestParentSingleChildFkOnParent(object):
         """Test sync delete and then sync in concurrent mode."""
         document = {
             "index": "testdb",
+            "database": "testdb",
             "nodes": {
                 "table": "book",
                 "columns": ["isbn", "title"],
